import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;
import javax.microedition.rms.*;
import java.util.*;

public class Game extends MIDlet implements CommandListener  
{ 
    static Command END_CMD = new Command("End", Command.BACK, 1);
    static Command QUIT_CMD = new Command("Quit", Command.SCREEN, 4);
    static Command HELP_CMD = new Command("Help", Command.HELP, 3);
    static Command SETTINGS_CMD = new Command("Settings", Command.SCREEN, 2);
    static Command START_CMD = new Command("Start", Command.BACK, 1);
    static Command BACK_CMD = new Command("Back", Command.BACK, 1);
    static Command SAVE_CMD = new Command("Save", Command.OK, 1);

    static final String STORE_NAME = "Mammoth.001";
    static final int    LOGO_TIMEOUT = 3000;

    String helpText = "Use [1-9] keys to choose a hunter and specify his direction with cursor keys";

    static Image logoImage;

    static { 
        try { 
            logoImage = Image.createImage("/logo.png");
        } catch (java.io.IOException err) {}
    }


    Form        main;
    RecordStore store;
    GameCanvas  game;
    Settings    settings;

    public Game() 
    {
        main = new Form("MammothHunter");
        main.append(new ImageItem(null, logoImage, ImageItem.LAYOUT_CENTER|ImageItem.LAYOUT_NEWLINE_AFTER, "Mammoth Hunter"));
        main.addCommand(START_CMD);
        main.addCommand(HELP_CMD);
        main.addCommand(SETTINGS_CMD);
        main.addCommand(QUIT_CMD);
    }

    protected void destroyApp(boolean unconditional) 
    {
        if (game != null) { 
            game.stop();
        }
        closeStore();
        main = null;
    }    

    protected  void pauseApp() 
    { 
        if (game != null) { 
            game.stop();
        }
    }

    protected void startApp() 
    {    
        Display display = Display.getDisplay(this);
	main.setCommandListener(this);
        display.setCurrent(main);
        if (store == null) { 
            openStore();
        }
    }


    private void openStore() {
        try { 
            try { 
                store = RecordStore.openRecordStore(STORE_NAME, false);
                loadSettings();
            } catch (RecordStoreNotFoundException x) { 
                store = RecordStore.openRecordStore(STORE_NAME, true);
                initializeSettings();
            }
        } catch(Exception x) { 
            settings = new Settings();
        }
    }

    private void closeStore() 
    {
        if (store != null) { 
            try { 
                store.closeRecordStore();
            } catch (Exception x) {}
        }
    }


    private void initializeSettings() {        
        settings = new Settings();
        byte[] record = settings.pack();
        try { 
            store.addRecord(record, 0, record.length);
        } catch (Exception x) {} 
    }
        
    private void loadSettings() {
        try { 
            settings = new Settings(store.getRecord(1));
        } catch (Exception x) {}
    }
        

    void saveSettings() { 
        byte[] record = settings.pack();
        try { 
            store.setRecord(1, record, 0, record.length);
        } catch (Exception x) {} 
    }

    public void commandAction(Command c, Displayable d) 
    {
        if (c == START_CMD) { 
            game = new GameCanvas(Display.getDisplay(this), main, settings);
            game.start();
        } else if (c == HELP_CMD) {
            Alert help = new Alert("Help", helpText, null, AlertType.INFO);
            help.setTimeout(Alert.FOREVER);
            Display.getDisplay(this).setCurrent(help, main);
        } else if (c == QUIT_CMD) { 
            destroyApp(true);
            notifyDestroyed();
        } else if (c == SETTINGS_CMD) { 
            new SettingsForm(this, main);
        }
    }
}



