import javax.microedition.lcdui.*;
import java.util.Random;

public class GameCanvas extends Canvas implements CommandListener {
    Display display;
    Displayable parent;
    Hunter[] hunters;
    int width, height;
    int nHunters;
    int delay;
    int current;
    int pitX;
    int pitY;
    int mammothX;
    int mammothY;
    int mammothDeltaX;
    int mammothDeltaY;
    int mammothSpeed; // related to the hunter speed
    int detectionDistance2;
    int mammothAbsentDelay;
    boolean repaintAll;
    boolean mammothRunning;
    boolean running;
    Random rand;

    static Image hunterImage;
    static Image mammothImage;
    static Image pitImage;
    static Image deadImage;

    static { 
        try { 
            hunterImage = Image.createImage("/hunter.png");
            mammothImage = Image.createImage("/mammoth.png");
            pitImage = Image.createImage("/pit.png");
            deadImage = Image.createImage("/dead.png");
        } catch (java.io.IOException err) {}
    }

    static final int WHITE = 0xffffff;

    static final int PIT_RADIUS = 3;
    static final int MAMMOTH_RADIUS = 4;
    static final int HUNTER_RADIUS = 2;
    static final int DELTA = 1;
    static final int FRACTION_BITS = 8;
    static final int MAMMOTH_ABSENT_DELAY = 100;
    
    public GameCanvas(Display display, Displayable parent, Settings settings) {
        this.display = display;
        this.parent = parent;
        nHunters = settings.nHunters;
        delay = Settings.MAX_SPEED - settings.hunterSpeed;
        mammothSpeed = (settings.mammothSpeed << FRACTION_BITS) / settings.hunterSpeed;
        
        hunters = new Hunter[Settings.MAX_HUNTERS];
	width = getWidth();
	height = getHeight();
        pitX = width / 2;
        pitY = height / 2;
        rand = new Random();
        detectionDistance2 = settings.detectionDistance*settings.detectionDistance;
        current = 0;
        int hunterX = width / 2 - nHunters*HUNTER_RADIUS; 
        int hunterY = height/2 + PIT_RADIUS + HUNTER_RADIUS*2;
        for (int i = 0; i < nHunters; i++) { 
            hunters[i] = new Hunter(hunterX, hunterY);
            hunterX += HUNTER_RADIUS*2;
        }
        mammothRunning = false;
        mammothAbsentDelay = MAMMOTH_ABSENT_DELAY;
        addCommand(Game.END_CMD);
        repaintAll = true;
    }

    void start() {         
        running = true;
        new TickThread().start();        
        setCommandListener(this);
        display.setCurrent(this);
    }
    
    void stop() { 
        running = false;
    }

    boolean tick() { 
        if (running) { 
            if (!mammothRunning) { 
                if (--mammothAbsentDelay < 0) { 
                    int mx, my;
                    int speed = mammothSpeed*DELTA;
                    int minDistanceToHunter;
                    do {
                        int v = rand.nextInt();
                        switch (v & 3) { 
                          case 0:
                            mx = MAMMOTH_RADIUS;
                            my = MAMMOTH_RADIUS + (v >>> 2) % (height-MAMMOTH_RADIUS*2);
                            mammothDeltaX = speed;
                            mammothDeltaY = 0;
                            break;
                          case 1:
                            mx = width - MAMMOTH_RADIUS;
                            my = MAMMOTH_RADIUS + (v >>> 2) % (height-MAMMOTH_RADIUS*2);
                            mammothDeltaX = -speed;
                            mammothDeltaY = 0;
                            break;
                          case 2:
                            mx = MAMMOTH_RADIUS + (v >>> 2) % (width-MAMMOTH_RADIUS*2);
                            my = MAMMOTH_RADIUS;
                            mammothDeltaX = 0;
                            mammothDeltaY = speed;
                            break;
                          default:
                            mx = MAMMOTH_RADIUS + (v >>> 2) % (width-MAMMOTH_RADIUS*2);
                            my = height - MAMMOTH_RADIUS;
                            mammothDeltaX = 0;
                            mammothDeltaY = -speed;
                        }
                        minDistanceToHunter = Integer.MAX_VALUE;
                        for (int i = 0, n = nHunters; i < n; i++) { 
                            Hunter h = hunters[i];
                            if (h != null) { 
                                int d = (mx-h.x)*(mx-h.x) + (my-h.y)*(my-h.y);
                                if (d < minDistanceToHunter) { 
                                    minDistanceToHunter = d;
                                }
                            }
                        }
                    } while (minDistanceToHunter <= (HUNTER_RADIUS+MAMMOTH_RADIUS+DELTA*4)*(HUNTER_RADIUS+MAMMOTH_RADIUS+DELTA*4)
                             || (my-pitY)*(my-pitY) <= (MAMMOTH_RADIUS+PIT_RADIUS)*(MAMMOTH_RADIUS+PIT_RADIUS)
                             || (mx-pitX)*(mx-pitX) <= (MAMMOTH_RADIUS+PIT_RADIUS)*(MAMMOTH_RADIUS+PIT_RADIUS));
                    mammothX = mx << FRACTION_BITS;
                    mammothY = my << FRACTION_BITS;
                    mammothRunning = true;
                }
            }
            repaint();
            return true;
        }
        return false;
    }

    static final int isqrt(int x) { 
        int a, b, m;
        a = 1;
        b = (x >>> 5) + 8;
        if (b > 65535) { 
            b = 65535;
        }
        do { 
            m = (a + b) >>> 1;
            if (m*m > x) { 
                b = m - 1;
            } else { 
                a = m + 1;
            }
        } while (b >= a);
        return a - 1;
    }

    class TickThread extends Thread { 
        public void run() { 
            try { 
                while (tick()) {
                    sleep(delay);
                }
            } catch (InterruptedException x) {}
        }
    }           

    protected void paint(Graphics g) {
        int n = nHunters;

        g.setColor(WHITE);
        if (repaintAll) { 
            g.fillRect(0, 0, width, height);
            g.drawImage(pitImage, pitX-PIT_RADIUS, pitY-PIT_RADIUS, Graphics.LEFT|Graphics.TOP);            
            for (int i = 0; i < n; i++) { 
                g.drawImage(hunterImage, hunters[i].x-HUNTER_RADIUS, hunters[i].y-HUNTER_RADIUS, Graphics.LEFT|Graphics.TOP);
            }
            repaintAll = false;
        }
      forEachHunter:
        for (int i = 0; i < n; i++) { 
            Hunter h = hunters[i];
            if (h == null || (h.dx == 0 && h.dy == 0)) { 
                continue;
            }
            int x = h.x + h.dx;
            int y = h.y + h.dy;
            if (x < HUNTER_RADIUS || x + HUNTER_RADIUS >= width
                || y < HUNTER_RADIUS || y + HUNTER_RADIUS >= height)
            {                    
                // hunter is near the margine - stop
                System.out.println("Hunter " + i + " near the margine");
                h.dx = 0;
                h.dy = 0;
                continue;
            }
            for (int j = 0; j < nHunters; j++) { 
                if  (i != j && hunters[j] != null) { 
                    if ((hunters[j].x - x)*(hunters[j].x - x) + 
                        (hunters[j].y - y)*(hunters[j].y - y) <= HUNTER_RADIUS*HUNTER_RADIUS*4)
                    {
                        // hunters are knocked - stop
                        h.dx = 0;
                        h.dy = 0;
                        System.out.println("Hunters " + i + " and " + j + " are knocked");
                        continue forEachHunter;
                    }
                }
            }
            
            g.fillRect(h.x-HUNTER_RADIUS, h.y-HUNTER_RADIUS, HUNTER_RADIUS*2, HUNTER_RADIUS*2);
            if ((x-pitX)*(x-pitX) + (y-pitY)*(y-pitY) <= (HUNTER_RADIUS+PIT_RADIUS)*(HUNTER_RADIUS+PIT_RADIUS)) 
            {
                // ohhh - hunter fall into the pit
                System.out.println("Hunters " + i + " fall into the pit");
                hunters[i] = null;
                continue;
            } 
            h.x = x;
            h.y = y;
            g.drawImage(hunterImage, x-HUNTER_RADIUS, y-HUNTER_RADIUS, Graphics.LEFT|Graphics.TOP);
        }
        if (mammothRunning) { 
            int dmx = 0;
            int dmy = 0;
            int mx = mammothX >>> FRACTION_BITS;
            int my = mammothY >>> FRACTION_BITS;

            for (int i = 0; i < n; i++) { 
                Hunter h = hunters[i];
                if (h == null) { 
                    continue;
                }
                int x = h.x;
                int y = h.y;
                int dx2 = (x-mx)*(x-mx);
                int dy2 = (y-my)*(y-my);
                int distance2 = dx2 + dy2;                                 
                
                if (distance2 <= (HUNTER_RADIUS+MAMMOTH_RADIUS+DELTA*2)*(HUNTER_RADIUS+MAMMOTH_RADIUS+DELTA*2)) 
                {
                    // ohhh - hunter hunter is killed by mammoth
                    g.drawImage(deadImage, x-HUNTER_RADIUS, y-HUNTER_RADIUS, Graphics.LEFT|Graphics.TOP);
                    System.out.println("Hunters " + i + " is killed by mammoth");
                    hunters[i] = null;
                    continue;
                } 
                if (distance2 <= detectionDistance2) {
                    int distance = isqrt(distance2);
                    dmx += ((mx - x) * detectionDistance2  << FRACTION_BITS) / distance2 / distance;
                    dmy += ((my - y) * detectionDistance2 << FRACTION_BITS) / distance2 / distance;
                }                
            }
            if (dmx != 0 || dmy != 0) { 
                int d = isqrt(dmx*dmx + dmy*dmy);
                mammothX += mammothDeltaX = dmx*DELTA*mammothSpeed/d;
                mammothY += mammothDeltaY =  dmy*DELTA*mammothSpeed/d;
            } else {
                mammothX += mammothDeltaX;
                mammothY += mammothDeltaY;
            }
            int x = mammothX >>> FRACTION_BITS;
            int y = mammothY >>> FRACTION_BITS;

            if (x != mx || y != my) {                
                g.fillRect(mx-MAMMOTH_RADIUS, my-MAMMOTH_RADIUS, MAMMOTH_RADIUS*2, MAMMOTH_RADIUS*2);
                if (x < MAMMOTH_RADIUS || x + MAMMOTH_RADIUS >= width
                    || y < MAMMOTH_RADIUS || y + MAMMOTH_RADIUS >= height)
                {                    
                    // mammoth run away
                    System.out.println("Mammoth run away");
                    mammothAbsentDelay = MAMMOTH_ABSENT_DELAY;
                    mammothRunning = false;        
                    return;
                } 
                if ((x-pitX)*(x-pitX) + (y-pitY)*(y-pitY) <= (MAMMOTH_RADIUS+PIT_RADIUS)*(MAMMOTH_RADIUS+PIT_RADIUS)) 
                {
                    // Ura!!! Mammoth in the pit
                    gameOver();
                    return;
                }
                g.drawImage(mammothImage, x-MAMMOTH_RADIUS, y-MAMMOTH_RADIUS, Graphics.LEFT|Graphics.TOP);
            }
        }
    }

    public void keyPressed(int keyCode) {
	switch (keyCode) {
          case KEY_NUM1:
          case KEY_NUM2:
          case KEY_NUM3:
          case KEY_NUM4:
          case KEY_NUM5:
          case KEY_NUM6:
          case KEY_NUM7:
          case KEY_NUM8:
          case KEY_NUM9:
            if (keyCode - KEY_NUM1 < nHunters) { 
                current = keyCode - KEY_NUM1;
            }            
            if (hunters[current] != null) { 
                hunters[current].dx = 0;
                hunters[current].dy = 0;    
            }       
            return;
          case KEY_STAR:
            if (hunters[current] != null) { 
                hunters[current].dx = -DELTA;
                hunters[current].dy = 0;
            }
            break;
          case KEY_POUND:
            if (hunters[current] != null) { 
                hunters[current].dx = DELTA;
                hunters[current].dy = 0;
            }
            break;
          default:
            switch (getGameAction(keyCode)) { 
              case LEFT:
                if (hunters[current] != null) { 
                    hunters[current].dx = -DELTA;
                    hunters[current].dy = 0;
                }
                break;
              case RIGHT:
                if (hunters[current] != null) { 
                    hunters[current].dx = DELTA;
                    hunters[current].dy = 0;
                }
                break;
              case UP:
                if (hunters[current] != null) { 
                    hunters[current].dx = 0;
                    hunters[current].dy = -DELTA;
                }
                break;
              case DOWN:
                if (hunters[current] != null) { 
                    hunters[current].dx = 0;
                    hunters[current].dy = DELTA;
                }
                break;
            }
        }
    }

    void gameOver() { 
        Alert a = new Alert("Mammoth in pit", "In this way all mammoths at the Earth were killed", null, AlertType.INFO);
        a.setTimeout(Alert.FOREVER);
        display.setCurrent(a, parent);
    }        

    public void commandAction(Command c, Displayable d)  
    {
        if (c == Game.END_CMD) {
            display.setCurrent(parent);
        }
    }
}    
